/* Construct w/h series from PSID data.
   First version: Oct. 2001.
   Merge 1983 to 1985 files. Pick wage and hours worked info */

libname  tempdir 'd:\elast_data\temp\';
filename fileref 'd:\elast_data\outdata\data84.asc';
filename infile0 'd:\psid\data68_93\68_93ind.dat';
filename infile1 'd:\psid\data83\83fam.dat';
filename infile2 'd:\psid\data84\84fam.dat';
filename infile3 'd:\psid\data85\85fam.dat';


    /* Read in 1983-1985 cross-year individual file and select variables
      needed for analysis. */

data tempdir.ind83_85(compress=no
                     rename=(V30399=ID83 V30429=ID84 V30463=ID85
                             V30402=AGE83 V30432=AGE84 V30466=AGE85
                             V30413=EDUC83 V32000=SEX));
infile infile0 LRECL=2189;

input
       V30001  2-5
       V30002  6-8
       V30399  919-922
       V30400  923-924
       V30401  925-926
       V30402  927-928
       V30413  948-949
       V30429  991-994
       V30430  995-996
       V30431  997-998
       V30432  999-1000
       V30463  1075-1078
       V30464  1079-1080
       V30465  1081-1082
       V30466  1083-1084
       V32000  2060;
label
       V30001 = "1968 INTERVIEW NUMBER 68"
       V30002 = "PERSON NUMBER         68"
       V30399 = "1983 INTERVIEW NUMBER"
       V30400 = "SEQUENCE NUMBER       83"
       V30401 = "RELATIONSHIP TO HEAD  83"
       V30402 = "AGE OF INDIVIDUAL     83"
       V30413 = "COMPLETED EDUCATION   83"
       V30429 = "1984 INTERVIEW NUMBER"
       V30430 = "SEQUENCE NUMBER       84"
       V30431 = "RELATIONSHIP TO HEAD  84"
       V30432 = "AGE OF INDIVIDUAL     84"
       V30463 = "1985 INTERVIEW NUMBER"
       V30464 = "SEQUENCE NUMBER       85"
       V30465 = "RELATIONSHIP TO HEAD  85"
       V30466 = "AGE OF INDIVIDUAL     85"
       V32000 = "SEX OF INDIVIDUAL";

/* Missing values */
IF V30402=99 THEN V30402=.;
IF V30413 IN(99,0) THEN V30413=.;
IF V30432=99 THEN V30432=.;
IF V30466=99 THEN V30466=.;

/* Select those men who were always heads 1988-1990 */
if (V32000 = 1) and
    (V30400 = 1 and V30401 = 10) and
    (V30430 = 1 and V30431 = 10) and
    (V30464 = 1 and V30465 = 10);

drop V30400 V30401 V30430 V30431 V30464 V30465;

run;


/* ======== 1983 ======== */

/* Read in 1983 family file and select variables needed for analysis */
data tempdir.fam83(compress=no
                  rename=(V8802=ID83 V9011=OCC83 V9005=EMPSTAT83
                          V9017=WH83 V8830=H83 V9276=MARSTAT83));
infile infile1 LRECL=1312;
input
       V8802   4-7
       V8830   70-73
       V8873   182-187
       V8875   189-193
       V8876   194-199
       V8877   200-204
       V8878   205-209
       V9005   591
       V9011   599-601
       V9017   615-618
       V9276   1034;
label
       V8802  = "1983 INTERVIEW NUMBER"
       V8830  = "HD ANN WRK HRS IN 82"
       V8873  = "HEAD 82 WAGES"
       V8875  = "HD BONUS/OT/COMM 82"
       V8876  = "HD PROF PRAC/TRADE 82"
       V8877  = "LABOR PT MKT GARDEN 82"
       V8878  = "LABOR PART ROOMERS 82"
       V9005  = "EMPLOYMENT STATUS 82-83"
       V9011  = "MAIN OCCUPATION 82-83"
       V9017  = "Wage per hour 82-83"
       V9276  = "Marital status 83";

/* Missing values */
if V9011 = 999 then V9011 = .;
if V8873 >= 999998 then V8873 = .;
if V8875 >= 99998 then V8875 = .;
if V8876 >= 999998 then V8876 = .;
if V8877 >= 99998 then V8877 = .;
if V8878 >= 99998 then V8878 = .;
if V9017 >= 9998 then V9017 = .;
run;

/* Sort files by id83 and merge */
proc sort data=tempdir.ind83_85 tagsort; by ID83;
proc sort data=tempdir.fam83    tagsort; by ID83;
run;

data tempdir.fam_ind(compress=yes);
merge tempdir.fam83 tempdir.ind83_85(in=ind83_85); by ID83; if ind83_85;
run;


/* ======== 1984 ======== */

/* Read in 1984 family file and select variables needed for analysis */
data tempdir.fam84(compress=no
                  rename=(V10002=ID84 V10899=REAL_ESTATE84 V10903=VEHICLES84
                          V10908=BUSINESS84 V10913=STOCKS84 V10918=ACCOUNTS84
                          V10923=BONDS84 V10933=DEBT84 V10018=HOUSE84 V10020=MORTGAGE84
                          V10453=EMPSTAT84 V10460=OCC84
                          V10466=WH84 V10037=H84 V10426=MARSTAT84));
infile infile2 LRECL=2000;
input
	   V10002  4-7
       V10018  30-35
       V10020  37-42
       V10037  87-90
       V10256  387-392
       V10258  394-398
       V10259  399-404
       V10260  405-409
       V10261  410-414
       V10426  839
       V10453  874
       V10460  881-883
       V10466  897-900
       V10899  1645-1651
       V10903  1655-1660
       V10908  1665-1671
       V10913  1676-1682
       V10918  1687-1692
       V10923  1697-1703
       V10933  1724-1729
       FAMSIZE 314-315
       FAMINC  1889-1895
       RACE    1953;
label
       V10002 = "1984 INTERVIEW NUMBER"
       V10018 = "House value 84"
       V10020 = "Remaining mortgage 84"
       V10037 = "HD ANN WRK HRS IN 83"
       V10256 = "HEAD 83 WAGES"
       V10258 = "HD BONUS/OT/COMM 83"
       V10259 = "HD PROF PRAC/TRADE 83"
       V10260 = "LABOR PT MKT GARDEN 83"
       V10261 = "LABOR PART ROOMERS 83"
       V10426 = "Marital status 84"
       V10453 = "EMPLOYMENT STATUS 83-84"
       V10460 = "MAIN OCCUPATION 83-84"
       V10466 = "Wage per hour 83-84"
       V10899 = "VALUE OF REAL ESTATE"
       V10903 = "...VEHICLES"
       V10908 = "...FARM/BUSINESS"
       V10913 = "...STOCKS"
       V10918 = "...ACCOUNTS"
       V10923 = "...BONDS/INSURANCE"
       V10933 = "...DEBT";
if V10460 = 999 then V10460 = .;
if V10256 >= 999998 then V10256 = .;
if V10258 >= 99998 then V10258 = .;
if V10259 >= 999998 then V10259 = .;
if V10260 >= 99998 then V10260 = .;
if V10261 >= 99998 then V10261 = .;
if V10466 >= 9998 then V10466 = .;
if V10018 >= 999998 then V10018 = .;
if V10020 >= 999998 then V10020 = .;
if V10899 >= 9999998 then V10899 = .; 
if V10903 >= 999998 then V10903 = .;
if V10908 >= 9999998 then V10908 = .;
if V10913 >= 9999998 then V10913 = .;
if V10918 >= 999998 then V10918 = .;
if V10923 >= 9999998 then V10923 = .;
if V10933 >= 999998 then V10933 = .;
run;

/* Sort files by id84 and merge */
proc sort data=tempdir.fam_ind tagsort; by ID84;
proc sort data=tempdir.fam84   tagsort; by ID84;
run;

data tempdir.fam_ind(compress=no);
merge tempdir.fam84 tempdir.fam_ind(in=fam_ind); by ID84; if fam_ind;
run;


/* ======== 1985 ======== */

/* Read in 1985 family file and select variables needed for analysis */
data tempdir.fam85(compress=no
                   rename=(V11102=ID85 V11637=EMPSTAT85 V11651=OCC85
                           V11657=WH85 V11146=H85 V11612=MARSTAT85));
infile infile3 LRECL=2456;
input
       V11102  4-7
       V11146  106-109
       V11397  437-442
       V11399  444-448
       V11400  449-454
       V11401  455-459
       V11402  460-464
       V11637  1039
       V11651  1056-1058
       V11657  1072-1075
       V11612  1005;
label
       V11102 = "1985 INTERVIEW NUMBER"
       V11146 = "HD ANN WRK HRS IN 84"
       V11397 = "HEAD 84 WAGES"
       V11399 = "HD BONUS/OT/COMM 84"
       V11400 = "HD PROF PRAC/TRADE 84"
       V11401 = "LABOR PT MKT GARDEN 84"
       V11402 = "LABOR PART ROOMERS 84"
       V11612 = "Marital status 85"
       V11637 = "EMPLOYMENT STATUS 84-85"
       V11651 = "MAIN OCCUPATION 84-85"
       V11657 = "Wage per hour 84-85";
if V11651 = 999 then V11651 = .;
if V11397 >= 999998 then V11397 = .;
if V11399 >= 99998 then V11399 = .;
if V11400 >= 999998 then V11400 = .;
if V11401 >= 99998 then V11401 = .;
if V11402 >= 99998 then V11402 = .;
if V11657 >= 9998 then V11657 = .;
run;

/* Sort files by id85 and merge */
proc sort data=tempdir.fam_ind tagsort; by ID85;
proc sort data=tempdir.fam85   tagsort; by ID85;
run;

data tempdir.fam_ind(compress=no);
merge tempdir.fam85 tempdir.fam_ind(in=fam_ind); by ID85; if fam_ind;
run;



/* Part 2 of PSID data manipulation.
   Generate series for hours worked and wage per hour. */


data tempdir.psid;
set tempdir.fam_ind;



/* ======== REMOVE OR REDEFINE MISSING VALUES ======== */

Y83 = V8873  + V8875  + V8876  + V8877  + V8878;
Y84 = V10256 + V10258 + V10259 + V10260 + V10261;
Y85 = V11397 + V11399 + V11400 + V11401 + V11402;

if AGE83>0 and AGE84>0 and AGE85>0;
if H83>0 and H84>0 and H85>0 and H83 < 5840 and H84 < 5840 and H85 < 5840;
if Y83>-999 and Y84>-999 and Y85>-999;

if WH83=. then WH83=-9;
if WH84=. then WH84=-9;
if WH85=. then WH85=-9;

if EMPSTAT83=. THEN EMPSTAT83=-9;
if EMPSTAT84=. THEN EMPSTAT84=-9;
if EMPSTAT85=. THEN EMPSTAT85=-9;
if MARSTAT83=. THEN MARSTAT83=-9;
if MARSTAT84=. THEN MARSTAT84=-9;
if MARSTAT85=. THEN MARSTAT85=-9;
if OCC83=. THEN OCC83=-9;
if OCC84=. THEN OCC84=-9;
if OCC85=. THEN OCC85=-9;
if EDUC83=. THEN EDUC83=-9;


if real_estate84=. then real_estate84=-999;
if vehicles84=. then vehicles84=-999;
if business84=. then business84=-999;
if stocks84=. then stocks84=-999;
if accounts84=. then accounts84=-999;
if bonds84=. then bonds84=-999;
if house84=. then house84=-999;
if mortgage84=. then mortgage84=-999;
if debt84=. then debt84=-999;


file fileref;
put @1   Y83   @11  Y84  @21  Y85
    @31  H83   @39  H84  @47  H85
    @55  WH83  @63  WH84  @71  WH85
    @79  AGE83 @83  AGE84 @87  AGE85
    @91  OCC83 @97  OCC84 @103 OCC85
    @109 EDUC83
    @115 REAL_ESTATE84 @125 VEHICLES84 @135 BUSINESS84
	@145 STOCKS84      @155 ACCOUNTS84 @165 BONDS84
    @175 HOUSE84       @185 MORTGAGE84 @195 DEBT84
    @205 EMPSTAT83     @210 EMPSTAT84  @215 EMPSTAT85 @220 SEX @225 V30001
    @235 MARSTAT83     @238 MARSTAT84  @241 MARSTAT85 @243 FAMSIZE @246 FAMINC
    @254 RACE;
run;
