function regress9395

chooseMARRIED = 0;
chooseWH      = 1;    % only include households with w**?
maxEMPSTAT    = 4;    % select only people with 'smaller' emp stat. 1=employed, 2=laid off, sick leave, 3= unemp, 4=...
minAGE        = 25;
maxAGE        = 60;
demDUM        = 0;    % demographic dummies?
yearDUM       = 0;    % year dummies?
yearDUM2nd    = 0;    % use dem dummies in second stage eqn?
instrumentW   = 1;    % use wage to instrument for wage?
instrumentWH  = 1;    % use other wage series to instrument?
explainWH     = 0;    % use other wage series as explanatory?
LEISURE       = 0;    % Estimate leisure elasticity rather than labor elast.?
TOTHOURS      = 24*365; % Total number of hours in a year
TOTHOURS      = 5000; % Total number of hours in a year
sameFIRST     = 1;    % Use same first stage equation for all subsamples?
chooseSRC     = 1;
STATS         = 0;    % Report summary statistics?
report.first  = 0;    % Report results from first stage regressions?
report.second = 0;    % Report results from second stage regressions?

load 'cpi.txt' -ascii

wh84 = load('data84.asc');
wh89 = load('data89.asc');
wh94 = load('data94.asc');

cpi84 = cpi([450 462 474],2);
cpi84 = cpi84 / cpi84(1);
cpi89 = cpi([510 522 534],2)/cpi(450,2);
cpi94 = cpi([570 582 594],2)/cpi(450,2);

% immediately delete households with missing information on 
% assets components or total family income
assets = wh84(:,[17:25 35]);
miss   = assets == -999;
keep   = find(sum(miss') == 0);
wh84   = wh84(keep,:);
assets = wh89(:,[17:25 35]);
miss   = assets == -999;
keep   = find(sum(miss') == 0);
wh89   = wh89(keep,:);
assets = wh94(:,[17:25 35]);
miss   = assets == -999;
keep   = find(sum(miss') == 0);
wh94   = wh94(keep,:);

n84           = size(wh84,1);
wh84(:,1:3)   = wh84(:,1:3) ./ (ones(n84,1)*cpi84');
wh84(:,7:9)   = wh84(:,7:9) ./ (ones(n84,1)*cpi84') / 100;
wh84(:,[17:25 35]) = wh84(:,[17:25 35]) / cpi84(2);


n89           = size(wh89,1);
wh89(:,1:3)   = wh89(:,1:3) ./ (ones(n89,1)*cpi89');
wh89(:,7:9)   = wh89(:,7:9) ./ (ones(n89,1)*cpi89') / 100;
wh89(:,[17:25 35]) = wh89(:,[17:25 35]) / cpi89(2);

n94           = size(wh94,1);
wh94(:,1:3)   = wh94(:,1:3) ./ (ones(n94,1)*cpi94');
wh94(:,7:9)   = wh94(:,7:9) ./ (ones(n94,1)*cpi94');
wh94(:,[17:25 35]) = wh94(:,[17:25 35]) / cpi94(2);
wh      = [wh84; wh89; wh94];


AGE     = wh(:,10:12);
INC     = wh(:,1:3);
H       = wh(:,4:6);
W       = INC./H;
WH      = wh(:,7:9);
FAMINC  = wh(:,35);              % total family income in 1983, 88, 93 (reported in 84, 89, 94)
FAMSIZE = wh(:,34);              % family size in 84, 89, 94
RACE    = wh(:,36) == 1;         % = 1 if white
n       = size(wh,1);
YEAR    = zeros(n,1);
YEAR(1:n84) = 1984;
YEAR(n84+1:n84+n89) = 1989;
YEAR(n84+n89+1:end) = 1994;


% ==========================================================================================
% REPORT STATISTICS
% ==========================================================================================
if STATS
fprintf(1,'=======================================================================================================\n');
fprintf(1,'Summary statistics, all data included (also miss vals), 1983 dollars:\n');
i84    = find(YEAR == 1984);
i89    = find(YEAR == 1989);
i94    = find(YEAR == 1994);
fprintf(1,'AGE:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(AGE(i84,:)),mean(AGE(i89,:)),mean(AGE(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(AGE(i84,:)),min(AGE(i89,:)),min(AGE(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(AGE(i84,:)),max(AGE(i89,:)),max(AGE(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n\n',...
        std(AGE(i84,:)),std(AGE(i89,:)),std(AGE(i94,:)));
fprintf(1,'INC:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(INC(i84,:)),mean(INC(i89,:)),mean(INC(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(INC(i84,:)),min(INC(i89,:)),min(INC(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(INC(i84,:)),max(INC(i89,:)),max(INC(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n\n',...
        std(INC(i84,:)),std(INC(i89,:)),std(INC(i94,:)));
fprintf(1,'H:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(H(i84,:)),mean(H(i89,:)),mean(H(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(H(i84,:)),min(H(i89,:)),min(H(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(H(i84,:)),max(H(i89,:)),max(H(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n\n',...
        std(H(i84,:)),std(H(i89,:)),std(H(i94,:)));
fprintf(1,'W:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(W(i84,:)),mean(W(i89,:)),mean(W(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(W(i84,:)),min(W(i89,:)),min(W(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(W(i84,:)),max(W(i89,:)),max(W(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n\n',...
        std(W(i84,:)),std(W(i89,:)),std(W(i94,:)));
fprintf(1,'WH:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(WH(i84,:)),mean(WH(i89,:)),mean(WH(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(WH(i84,:)),min(WH(i89,:)),min(WH(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(WH(i84,:)),max(WH(i89,:)),max(WH(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n\n',...
        std(WH(i84,:)),std(WH(i89,:)),std(WH(i94,:)));

fprintf(1,'-------------------------------------------------------------------------------------------------------\n');
fprintf(1,'Summary statistics, missing values excluded, 1983 dollars, only non-zero values:\n');
i84    = find(YEAR == 1984 & W(:,1) > 0 & W(:,2) > 0 & W(:,3) > 0);
i89    = find(YEAR == 1989 & W(:,1) > 0 & W(:,2) > 0 & W(:,3) > 0);
i94    = find(YEAR == 1994 & W(:,1) > 0 & W(:,2) > 0 & W(:,3) > 0);
fprintf(1,'W:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(W(i84,:)),mean(W(i89,:)),mean(W(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(W(i84,:)),min(W(i89,:)),min(W(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(W(i84,:)),max(W(i89,:)),max(W(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        std(W(i84,:)),std(W(i89,:)),std(W(i94,:)));
fprintf(1,'n    %10d %10d %10d %10d %10d %10d %10d %10d %10d\n\n',...
        length(i84)*[1 1 1],length(i89)*[1 1 1],length(i94)*[1 1 1]);
i84    = find(YEAR == 1984 & WH(:,1) > 0 & WH(:,2) > 0 & WH(:,3) > 0);
i89    = find(YEAR == 1989 & WH(:,1) > 0 & WH(:,2) > 0 & WH(:,3) > 0);
i94    = find(YEAR == 1994 & WH(:,1) > 0 & WH(:,2) > 0 & WH(:,3) > 0);
fprintf(1,'WH:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(WH(i84,:)),mean(WH(i89,:)),mean(WH(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(WH(i84,:)),min(WH(i89,:)),min(WH(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(WH(i84,:)),max(WH(i89,:)),max(WH(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        std(WH(i84,:)),std(WH(i89,:)),std(WH(i94,:)));
fprintf(1,'n    %10d %10d %10d %10d %10d %10d %10d %10d %10d\n\n',...
        length(i84)*[1 1 1],length(i89)*[1 1 1],length(i94)*[1 1 1]);
end
% ==========================================================================================




dAGE2   = AGE(:,2)-AGE(:,1);
dAGE3   = AGE(:,3)-AGE(:,2);
AGE     = AGE(:,2);

warning off MATLAB:divideByZero
dW1   = W(:,2)./W(:,1);
dW2   = W(:,3)./W(:,2);
warning on MATLAB:divideByZero

if chooseWH
  warning off MATLAB:divideByZero
  dWH1  = WH(:,2)./WH(:,1);
  dWH2  = WH(:,3)./WH(:,2);
  warning on MATLAB:divideByZero
end

dH1   = H(:,2)./H(:,1);
dH2   = H(:,3)./H(:,2);

if chooseSRC,     testSRC=5000; else testSRC=99999; end
if chooseMARRIED, testMAR=2; else    testMAR=99999; end
if chooseWH
       keepWH = WH(:,1)>0.50  & WH(:,2)>0.50 & WH(:,3)>0.50 & ...
       dWH1<3.5 & dWH2<3.5 & dWH1>0.40 & dWH2>0.40;
else keepWH = ones(size(WH(:,1))); 
end

EMP = wh(:,26:28);    % employment status
MAR = wh(:,31:33);    % marital status

keep = find(AGE<=maxAGE & AGE>=minAGE & dAGE2>=0 & dAGE2<3 & dAGE3>=0 &dAGE3<3 & ...
          min(INC(:,1:3)')'>0 & ...
          dW1<3.5 & dW2<3.5 & ...
          dW1>0.40 & dW2>0.40 & ...
          dH1<3.5 & dH2<3.5 & ...
          dH1>0.40 & dH2>0.40 & ...
          H(:,1)>0 & H(:,2)>0 & H(:,3)>0 & ...
          H(:,1)<4861 & H(:,2)<4861 & H(:,3)<4861 & ...
          keepWH & ...
          EMP(:,1)<maxEMPSTAT & EMP(:,2)<maxEMPSTAT & EMP(:,3)<maxEMPSTAT & ...
          wh(:,30)<testSRC & MAR(:,1)<testMAR & MAR(:,2)<testMAR & MAR(:,3)<testMAR & ...
          EMP(:,1)>0 & EMP(:,2)>0 & EMP(:,3)>0 & ...
          MAR(:,1)>0 & MAR(:,2)>0 & MAR(:,3)>0 & ...
          wh(:,16)>0);
            
n84_keep = length(find(YEAR(keep)==1984));
n89_keep = length(find(YEAR(keep)==1989));
n94_keep = length(find(YEAR(keep)==1994));
DUM84    = YEAR(keep) == 1984; 
DUM89    = YEAR(keep) == 1989; 

AGE     = AGE(keep,:);
INC     = INC(keep,:);
H       = H(keep,:);
W       = W(keep,:);
WH      = WH(keep,:);
YEAR    = YEAR(keep);
FAMINC  = FAMINC(keep);
FAMSIZE = FAMSIZE(keep);
RACE    = RACE(keep);


[OCC, OCC_low, OCC_hgh] = occupation(wh(keep,13));
EDUC = wh(keep,16);
nn   = length(keep);


% Construct wealth variable (='cash')
allA  = sum(wh(keep,17:23)')'-sum(wh(keep,24:25)')';
liqA  = sum(wh(keep,20:22)')'-wh(keep,25);
hseA  = wh(keep,23)-wh(keep,24);
Debt  = sum(wh(keep,24:25)')';
House = wh(keep,23);
savA  = sum(wh(keep,21:22)')';
savB  = sum(wh(keep,20:22)')';

% ==========================================================================================
% REPORT STATISTICS
% ==========================================================================================
if STATS
i84    = find(YEAR == 1984);
i89    = find(YEAR == 1989);
i94    = find(YEAR == 1994);
fprintf(1,'=======================================================================================================\n');
fprintf(1,'Summary statistics, selected data:\n');
fprintf(1,'     %10d %10d %10d %10d %10d %10d %10d %10d %10d\n',...    
        1983:1985,1988:1990,1993:1995);
fprintf(1,'INC:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(INC(i84,:)),mean(INC(i89,:)),mean(INC(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(INC(i84,:)),min(INC(i89,:)),min(INC(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(INC(i84,:)),max(INC(i89,:)),max(INC(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n\n',...
        std(INC(i84,:)),std(INC(i89,:)),std(INC(i94,:)));
fprintf(1,'H:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(H(i84,:)),mean(H(i89,:)),mean(H(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(H(i84,:)),min(H(i89,:)),min(H(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(H(i84,:)),max(H(i89,:)),max(H(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n\n',...
        std(H(i84,:)),std(H(i89,:)),std(H(i94,:)));
fprintf(1,'W:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(W(i84,:)),mean(W(i89,:)),mean(W(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(W(i84,:)),min(W(i89,:)),min(W(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(W(i84,:)),max(W(i89,:)),max(W(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n\n',...
        std(W(i84,:)),std(W(i89,:)),std(W(i94,:)));
fprintf(1,'WH:  \nMean %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        mean(WH(i84,:)),mean(WH(i89,:)),mean(WH(i94,:)));
fprintf(1,'Min  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        min(WH(i84,:)),min(WH(i89,:)),min(WH(i94,:)));
fprintf(1,'Max  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n',...
        max(WH(i84,:)),max(WH(i89,:)),max(WH(i94,:)));
fprintf(1,'Std  %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f %10.2f\n\n',...
        std(WH(i84,:)),std(WH(i89,:)),std(WH(i94,:)));

fprintf(1,'     %12d %12d %12d\n',1984,1989,1994);
fprintf(1,'n    %12d %12d %12d\n\n',...
        length(i84),length(i89),length(i94));
fprintf(1,'AGE:  \nMean %12.2f %12.2f %12.2f\n',...
        mean(AGE(i84)),mean(AGE(i89)),mean(AGE(i94)));
fprintf(1,'Min  %12.2f %12.2f %12.2f\n',...
        min(AGE(i84)),min(AGE(i89)),min(AGE(i94)));
fprintf(1,'Max  %12.2f %12.2f %12.2f\n',...
        max(AGE(i84)),max(AGE(i89)),max(AGE(i94)));
fprintf(1,'Std  %12.2f %12.2f %12.2f\n\n',...
        std(AGE(i84)),std(AGE(i89)),std(AGE(i94)));
fprintf(1,'Net total wealth: \nMean %12.2f %12.2f %12.2f\n',...
        mean(allA(i84)),mean(allA(i89)),mean(allA(i94)));
fprintf(1,'Min  %12.2f %12.2f %12.2f\n',...
        min(allA(i84)),min(allA(i89)),min(allA(i94)));
fprintf(1,'Max  %12.2f %12.2f %12.2f\n',...
        max(allA(i84)),max(allA(i89)),max(allA(i94)));
fprintf(1,'Std  %12.2f %12.2f %12.2f\n\n',...
        std(allA(i84)),std(allA(i89)),std(allA(i94)));
fprintf(1,'Net liquid wealth: \nMean %12.2f %12.2f %12.2f\n',...
        mean(liqA(i84)),mean(liqA(i89)),mean(liqA(i94)));
fprintf(1,'Min  %12.2f %12.2f %12.2f\n',...
        min(liqA(i84)),min(liqA(i89)),min(liqA(i94)));
fprintf(1,'Max  %12.2f %12.2f %12.2f\n',...
        max(liqA(i84)),max(liqA(i89)),max(liqA(i94)));
fprintf(1,'Std  %12.2f %12.2f %12.2f\n\n',...
        std(liqA(i84)),std(liqA(i89)),std(liqA(i94)));
fprintf(1,'Net house value: \nMean %12.2f %12.2f %12.2f\n',...
        mean(hseA(i84)),mean(hseA(i89)),mean(hseA(i94)));
fprintf(1,'Min  %12.2f %12.2f %12.2f\n',...
        min(hseA(i84)),min(hseA(i89)),min(hseA(i94)));
fprintf(1,'Max  %12.2f %12.2f %12.2f\n',...
        max(hseA(i84)),max(hseA(i89)),max(hseA(i94)));
fprintf(1,'Std  %12.2f %12.2f %12.2f\n\n',...
        std(hseA(i84)),std(hseA(i89)),std(hseA(i94)));
   
end
% ==========================================================================================


if LEISURE
  H   = TOTHOURS - H;
end

w     = log(W);

if chooseWH
  w2  = log(WH);
else 
  w2  = w;
end


dw2   = w(:,2)-w(:,1);
dw3   = w(:,3)-w(:,2);

dwh2  = w2(:,2)-w2(:,1);
dwh3  = w2(:,3)-w2(:,2);


% Note: Japelli's data is in 1982 dollars, our data is in 1983 dollar --> no rescaling necessary
EMP     = EMP(keep,2);
MAR     = MAR(keep,2);
logY    = log(FAMINC);
logA    = log(max(allA,exp(0)));
logD    = log(max(Debt,exp(0)));
UNEMP   = EMP >= 2;
MARRIED = MAR == 1;
House   = House > 0;
Sav     = liqA > 0;
p       = 1.927*logY - 0.102*logY.^2 + 0.518*logA - 0.0043*logA.^2 - 0.051*logA.*logY + ...
          0.026*logD + 0.014*AGE -0.0007*AGE.^2 + 0.0020*AGE.*logY - 0.0014*AGE.*logA - ...
          0.002*max(EDUC,0) + 0.214*UNEMP - 0.267*MARRIED - 0.427*RACE + ...
          0.083*FAMSIZE - 0.364*House - 0.240*Sav - 9.122;
      

% ====================================================
% Regressions
%
% Let Y  = dependent variable
%     X1 = exogenous explanatory
%     X2 = endogenous explanatory
%     X  = [X1 X2]
%     Z0 = instruments for X2
%     Z  = instruments for X 
% ====================================================

Y   = log(H(:,3)) - log(H(:,2));
X1  = ones(nn,1);

if explainWH
    X2 = dwh3;
else
    X2 = dw3;
end

if instrumentW
  if instrumentWH
       Z0 = [dwh2 w2(:,2)];
  else Z0 = [dw2 w(:,2)];  
  end
else
   Z0 = [];
end

EXTRA_VARS = [];
if demDUM
%  EXTRA_VARS = [EXTRA_VARS AGE AGE.^2 EDUC EDUC.^2 AGE.*EDUC AGE.*(EDUC.^2) OCC_low.*AGE OCC_low OCC_hgh.*AGE OCC_hgh.*AGE.^2];
%  EXTRA_VARS = [EXTRA_VARS AGE AGE.^2 OCC_hgh OCC_hgh.*AGE OCC_hgh.*AGE.^2];
   EXTRA_VARS = [EXTRA_VARS AGE AGE.^2 EDUC EDUC.^2 AGE.*EDUC AGE.*EDUC.^2];
end

if yearDUM2nd
  X1 = [X1 DUM84 DUM89];
elseif yearDUM
  EXTRA_VARS = [EXTRA_VARS DUM84 DUM89];
end

Z0 = [Z0 EXTRA_VARS];



report.title = '>>> All included <<<\n';
Ret(1) = tsls(Y,X1,X2,Z0,[],report);
if sameFIRST
  Xhat = Ret(1).Xhat;
  report.first = 0;
end

ii = find(liqA>0);
report.title = '>>> Liquid wealth > 0 <<<\n';
if sameFIRST, Ret(2) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),Xhat(ii,:),report);
else Ret(2) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),[],report); end

ii = find(liqA>(0.5*160*W(:,2)));
report.title = '>>> Liquid wealth > 0.5 monthly <<<\n';
if sameFIRST, Ret(3) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),Xhat(ii,:),report);
else Ret(3) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),[],report); end

ii = find(liqA>(1*160*W(:,2)));
report.title = '>>> Liquid wealth > 1 monthly <<<\n';
if sameFIRST, Ret(4) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),Xhat(ii,:),report);
else Ret(4) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),[],report); end
selectLIQ = 0*Y;
selectLIQ(ii) = 1;

ii = find(liqA>1000);
report.title = '>>> Liquid wealth > $1,000 <<<\n';
if sameFIRST, Ret(5) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),Xhat(ii,:),report);
else Ret(5) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),[],report); end

ii = find(allA>0);
report.title = '>>> Total wealth > 0 <<<\n';
if sameFIRST, Ret(6) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),Xhat(ii,:),report);
else Ret(6) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),[],report); end

ii = find(allA>(0.5*160*12*W(:,2)));
report.title = '>>> Total wealth > 0.5 yearly <<<\n';
if sameFIRST, Ret(7) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),Xhat(ii,:),report);
else Ret(7) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),[],report); end

ii = find(allA>(1*160*12*W(:,2)));
report.title = '>>> Total wealth > 1 yearly <<<\n';
if sameFIRST, Ret(8) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),Xhat(ii,:),report);
else Ret(8) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),[],report); end
selectALL = 0*Y;
selectALL(ii) = 1;

ii = find(allA>12000);
report.title = '>>> Total wealth > $12,000 <<<\n';
if sameFIRST, Ret(9) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),Xhat(ii,:),report);
else Ret(9) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),[],report); end

ii = find(p<median(p));
ii = find(p<-1.432706);
pp=1-1./(1+exp(p));
%ii = find(pp<0.25);
%ii = find(pp<median(pp));
ii = find(pp<0.192678);
report.title = '>>> p < median(p) <<<\n';
if sameFIRST, Ret(10) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),Xhat(ii,:),report);
else Ret(10) = tsls(Y(ii),X1(ii,:),X2(ii,:),Z0(ii,:),[],report); end

fprintf(1,'\n===============================================================================\n');
fprintf(1,'>>>>>             REGRESS 1983-1985, 1988-1990, 1993-1995                 <<<<<\n')
fprintf(1,'                    %s ',date); cl = clock;
fprintf(1,'%2d:%2d\n',cl(4:5));
fprintf(1,'MARRIED  = %6d   ',chooseMARRIED);
fprintf(1,'SAME 1st = %6d\n',sameFIRST);
fprintf(1,'#83-85   = %6d   ',n84_keep);
fprintf(1,'#88-90   = %6d\n',n89_keep);
fprintf(1,'#93-95   = %6d\n',n94_keep);


fprintf(1,'\n========================================================================================\n');
fprintf(1,'           no       0      0.5      1      1000      0     0.5      1     12000   low p\n');
fprintf(1,'----------------------------------------------------------------------------------------\n');
if sameFIRST
fprintf(1,'R2(1st) %6.2f\n',Ret(1).FstR2);
fprintf(1,'F(1st)  %6.2f\n',Ret(1).FstF);
fprintf(1,'Sig(1st)%6s\n',Ret(1).FstSig);
fprintf(1,'*** Warning: Second stage F values not correctly calculated in subsamples (?) *** \n');
else
fprintf(1,'R2(1st) %6.2f   %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f\n',[Ret.FstR2]);
fprintf(1,'F(1st)  %6.2f   %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f\n',[Ret.FstF]);
fprintf(1,'Sig(1st)%6s   %6s  %6s  %6s  %6s  %6s  %6s  %6s  %6s  %6s\n',Ret(:).FstSig);
end
fprintf(1,'----------------------------------------------------------------------------------------\n');
fprintf(1,'const   %6.2f   %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f\n',[Ret.const]);
fprintf(1,'         (%4.2f)   (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)\n',...
        [Ret.const_std]);
fprintf(1,'dw      %6.2f   %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f\n',[Ret.dw]);
fprintf(1,'         (%4.2f)   (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)  (%4.2f)\n',...
        [Ret.dw_std]);
fprintf(1,'----------------------------------------------------------------------------------------\n');
fprintf(1,'R2      %6.2f   %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f\n',[Ret.R2]);
fprintf(1,'F       %6.2f   %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f  %6.2f\n',[Ret.F]);
fprintf(1,'Sig     %6s   %6s  %6s  %6s  %6s  %6s  %6s  %6s  %6s  %6s\n',Ret(:).sig);
fprintf(1,'n       %6d   %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d  %6d\n',[Ret.n]);




% ====================================================================================
% FUNCTION OCCUPATION
%
% Create occupation dummies
%
% ====================================================================================
function [occdum, occlow, occhgh] = occupation(occ);

occdum(:,1) = ((occ >= 1) & (occ <= 195));     % Prof., Techn. Workers
occdum(:,2) = ((occ >= 201) & (occ <= 245));   % Managers
occdum(:,3) = ((occ >= 260) & (occ <= 285));   % Sales workers
occdum(:,4) = ((occ >= 301) & (occ <= 395));   % Clerical workers
occdum(:,5) = ((occ >= 401) & (occ <= 600));   % Craftsmen
occdum(:,6) = ((occ >= 601) & (occ <= 715));   % Operatives
occdum(:,7) = ((occ >= 801) & (occ <= 824));   % Farmers, Farm laborers
occdum(:,8) = ((occ >= 901) & (occ <= 984));   % Service workers
occdum(:,9) = 1 - sum(occdum(:,1:8)')'; % Other


occlow = occdum(:,3) + occdum(:,4) + occdum(:,5) + occdum(:,7) + occdum(:,8);
occhgh = occdum(:,1) + occdum(:,2);



% ====================================================================================
% FUNCTION tsls
%
% Two stage least squares
%
%     Y      = dependent variable
%     X1     = exogenous explanatory
%     X2     = endogenous explanatory
%     Z0     = instruments for X2
%     XHAT   = empty or instruments calculated in larger sample
%     report = struct used to transport information
%
% ====================================================================================
function ret = tsls(Y,X1,X2,Z0,Xhat,report)

Fstat99 = [6.63 4.61 3.78 3.32 3.02 2.80 2.64 2.51 2.41 2.32 2.25 2.18 2.13 2.09 2.04 2.01 1.98 1.95 1.91 1.88:-0.018:1.70];
Fstat95 = [3.84 3.00 2.60 2.37 2.21 2.10 2.01 1.94 1.88 1.83 1.79 1.75 1.72 1.69 1.67 1.65 1.63 1.61 1.59 1.57:-0.011:1.46];

X = [X1 X2];
if isempty(Xhat)
    Z    = [X1 Z0];
else
    Z    = Xhat;
end
T = size(Y,1);
k = size(X,2);

bz   = inv(Z'*Z)*Z'*X;
Xhat = Z*bz;


% Second stage
b    = inv(Xhat'*X)*Xhat'*Y;
u    = Y - X*b;

Yhat = Xhat*b;
SSE  = (u-mean(u))'*(u-mean(u));
SSR  = (Yhat-mean(Yhat))'*(Yhat-mean(Yhat));

s2   = SSE /(T-k);
S    = s2*inv(Xhat'*X);
stdb = sqrt(diag(S));

if report.second
fprintf(1,'==============================================\n')
if isfield(report,'title'), fprintf(1,report.title), end
end

n = size(X2,2);
for i = 1:n
    if report.first
    fprintf(1,'First stage regression:\n')
    if n>1, fprintf(1,'Endogenous variable #%2d\n',i); end
    end
    y1 = X2(:,i);
    b1 = inv(Z'*Z)*Z'*y1;
    e1 = y1 - Z*b1;
    k1 = size(Z,2);
    ss = sum(e1.^2)/(T-k1);
    SS = ss*inv(Z'*Z);
    stdb1 = sqrt(diag(SS));
    rr = 1 - e1'*e1 / ((y1-mean(y1))'*(y1-mean(y1)));
    ff = (rr/(k1-1)) / ((1-rr)/(T-k1));
    if ff > Fstat99(k1-1), sig = '**';
    elseif ff > Fstat95(k1-1), sig = '* '; else sig = '  '; end;
    if report.first
    fprintf(1,'R2 = %8.3f   ',rr);
    fprintf(1,'F  = %7.2f ',ff), fprintf(1,'%2s\n',sig);
    fprintf(1,'%12.6f   (%10.6f)\n',[b1,stdb1]');
    fprintf(1,'----------------------------------------------\n')
    end
    ret.FstSig = sig;
end

r2    = 1 - u'*u / ((Y-mean(Y))'*(Y-mean(Y)));                       % As reported by EViews
r2    = SSR / (SSR+SSE);                                             % As reported by SAS and SPSS. Fstat should be based ...
fstat = (r2/(k-1)) / ((1-r2)/(T-k));                                 % ... on this r2
if fstat > Fstat99(k-1), sig = '**';
elseif fstat > Fstat95(k-1), sig = '* '; else sig = '  '; end;
if report.second
fprintf(1,'R2 = %8.3f\n',r2);                                          
fprintf(1,'F  = %6.2f ',fstat), fprintf(1,'%2s   ',sig);
fprintf(1,'n = %6d\n',T);
fprintf(1,'%12.6f   (%10.6f)\n',[b stdb]');
end

ret.const = b(1);
ret.const_std = stdb(1);
ret.dw    = b(end);
ret.dw_std    = stdb(end);
ret.R2    = r2;
ret.F     = fstat;
ret.n     = T;
ret.sig   = sig;
ret.FstR2 = rr;
ret.FstF  = ff;
ret.Xhat  = Xhat;